@@ -16,9 +16,48 @@ NormalizingWeight <- function(A, normalize=FALSE){
  stop("normalize argument not recognized in NormalizingWeight")
 }
 
-GPForth <- function(A, Tmat=diag(ncol(A)), method="varimax",
-                    normalize=FALSE, eps=1e-5, maxit=1000, ...){
- #   previously eps=1e-5
+print.GPArotation <- function (x, digits=3, Table=FALSE, ...){
+   cat(if(x$orthogonal)"Orthogonal" else "Oblique")
+   cat(" rotation method", x$method)
+   cat((if(!x$convergence)" NOT" ), "converged.\n")
+
+   cat("Loadings:\n")
+   print(x$Lh, digits=digits)
+
+   cat("\nRotating matrix:\n")
+   print(t(solve(x$Th)), digits=digits)
+
+   if(!x$orthogonal){
+     cat("\nPhi:\n")
+     print(x$Phi, digits=digits)
+     }
+
+   if(Table){
+     cat("\nIteration table:\n")
+     print(x$Table, digits=digits)
+     }
+   invisible(x, ...)
+   }
+
+summary.GPArotation <- function (object, ...){ 
+   r <- list(Lh=object$Lh, method=object$method, orthogonal=object$orthogonal, 
+          convergence=object$convergence, iters=nrow(object$Table))
+   class(r) <- "summary.GPArotation"
+   r
+   }
+
+print.summary.GPArotation <- function (x, digits=3, ...){
+   cat(if(x$orthogonal)"Orthogonal" else "Oblique")
+   cat(" rotation method", x$method)
+   if(!x$convergence) cat(" NOT" )
+   cat(" converged in ", x$iter, " iterations.\n")
+
+   cat("Loadings:\n")
+   print(x$Lh, digits=digits)
+  }
+
+GPForth <- function(A, Tmat=diag(ncol(A)), normalize=FALSE, eps=1e-5, maxit=1000, 
+		    method="varimax", methodArgs=NULL){
  if((!is.logical(normalize)) || normalize) {
      W <- NormalizingWeight(A, normalize=normalize)
      normalize <- TRUE
@@ -27,8 +66,10 @@ GPForth <- function(A, Tmat=diag(ncol(A)), method="varimax",
  if(1 >= ncol(A)) stop("rotation does not make sense for single factor models.")
  al <- 1
  L <- A %*% Tmat
- Method <- get(paste("vgQ",method,sep="."))
- VgQ <- Method(L,...)
+ #Method <- get(paste("vgQ",method,sep="."))
+ #VgQ <- Method(L, ...)
+ Method <- paste("vgQ",method,sep=".")
+ VgQ <- do.call(Method, append(list(L), methodArgs))
  G <- crossprod(A,VgQ$Gq)
  f <- VgQ$f
  Table <- NULL
@@ -45,7 +86,8 @@ GPForth <- function(A, Tmat=diag(ncol(A)), method="varimax",
      UDV <- svd(X)
      Tmatt <- UDV$u %*% t(UDV$v)
      L <- A %*% Tmatt
-     VgQt <- Method(L,...)
+     #VgQt <- Method(L, ...)
+     VgQt <- do.call(Method, append(list(L), methodArgs))
      if (VgQt$f < (f - 0.5*s^2*al)) break
      al <- al/2
      }
@@ -58,13 +100,14 @@ GPForth <- function(A, Tmat=diag(ncol(A)), method="varimax",
      warning("convergence not obtained.", maxit, " iterations used.")
  if(normalize) L <- L * W
  dimnames(L) <- dimnames(A)
- list(Lh=L, Th=Tmat, Table=Table, 
-      method=VgQ$Method, orthogonal=TRUE, convergence=convergence)
+ r <- list(Lh=L, Th=Tmat, Table=Table, 
+        method=VgQ$Method, orthogonal=TRUE, convergence=convergence)
+ class(r) <- "GPArotation"
+ r
 }
 
-GPFoblq <- function(A, Tmat=diag(ncol(A)), method="quartimin",
-                    normalize=FALSE, eps=1e-5, maxit=1000, ...){
- #   previously eps=1e-5
+GPFoblq <- function(A, Tmat=diag(ncol(A)), normalize=FALSE, eps=1e-5, maxit=1000, 
+		    method="quartimin", methodArgs=NULL){
  if(1 >= ncol(A)) stop("rotation does not make sense for single factor models.")
  if((!is.logical(normalize)) || normalize) {
      W <- NormalizingWeight(A, normalize=normalize)
@@ -73,8 +116,10 @@ GPFoblq <- function(A, Tmat=diag(ncol(A)), method="quartimin",
      }
  al <- 1
  L <- A %*% t(solve(Tmat))
- Method <- get(paste("vgQ",method,sep="."))
- VgQ <- Method(L, ...)
+ #Method <- get(paste("vgQ",method,sep="."))
+ #VgQ <- Method(L, ...)
+ Method <- paste("vgQ",method,sep=".")
+ VgQ <- do.call(Method, append(list(L), methodArgs))
  G <- -t(t(L) %*% VgQ$Gq %*% solve(Tmat))
  f <- VgQ$f
  Table <- NULL
@@ -90,7 +135,8 @@ GPFoblq <- function(A, Tmat=diag(ncol(A)), method="quartimin",
      v <- 1/sqrt(c(rep(1,nrow(X)) %*% X^2))
      Tmatt <- X %*% diag(v)
      L <- A %*% t(solve(Tmatt))
-     VgQt <- Method(L,...)
+     #VgQt <- Method(L, ...)
+     VgQt <- do.call(Method, append(list(L), methodArgs))
      improvement <- f - VgQt$f 
      if (improvement >  0.5*s^2*al) break
      al <- al/2
@@ -104,15 +150,18 @@ GPFoblq <- function(A, Tmat=diag(ncol(A)), method="quartimin",
      warning("convergence not obtained.", maxit, " iterations used.")
  if(normalize) L <- L * W
  dimnames(L) <- dimnames(A)
- list(Lh=L, Phi=t(Tmat) %*% Tmat, Th=Tmat, Table=Table,
+ r <- list(Lh=L, Phi=t(Tmat) %*% Tmat, Th=Tmat, Table=Table,
       method=VgQ$Method, orthogonal=FALSE, convergence=convergence)
+ class(r) <- "GPArotation"
+ r
 }
 
 #######################
 
 
-oblimin <- function(L, Tmat=diag(ncol(L)), gam=0, normalize=FALSE, eps=1e-8, maxit=1000){
-   z <- GPFoblq(L, Tmat=Tmat, method="oblimin", gam=gam, normalize=normalize, eps=eps, maxit=maxit)
+oblimin <- function(L, Tmat=diag(ncol(L)), gam=0, normalize=FALSE, eps=1e-5, maxit=1000){
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
+                method="oblimin",  methodArgs=list(gam=gam))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -147,7 +196,7 @@ vgQ.oblimin <- function(L, gam=0){
 #vgQ.oblimin(FA2)$Gq - vgQ.origoblimin(FA2)$Gq
 
 
-quartimin <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000){
+quartimin <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000){
    z <- GPFoblq(L, Tmat=Tmat, method="quartimin", normalize=normalize, eps=eps, maxit=maxit)
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
@@ -172,15 +221,17 @@ vgQ.quartimin <- function(L){
 #} 
 
 
-targetT <- function(L, Tmat=diag(ncol(L)), Target=NULL, normalize=FALSE, eps=1e-8, maxit=1000) {
+targetT <- function(L, Tmat=diag(ncol(L)), Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
    if(is.null(Target)) stop("argument Target must be specified.")
-   z <- GPForth(L, Tmat=Tmat, method="target",Target=Target, normalize=normalize, eps=eps, maxit=maxit)
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+           method="target", methodArgs=list(Target=Target))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
-targetQ <- function(L, Tmat=diag(ncol(L)), Target=NULL, normalize=FALSE, eps=1e-8, maxit=1000) {
+targetQ <- function(L, Tmat=diag(ncol(L)), Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
    if(is.null(Target)) stop("argument Target must be specified.")
-   z <- GPFoblq(L, Tmat=Tmat, method="target",Target=Target, normalize=normalize, eps=eps, maxit=maxit)
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+           method="target", methodArgs=list(Target=Target))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -192,17 +243,19 @@ vgQ.target <- function(L, Target=NULL){
 	Method="Target rotation")
    }
 
-pstT <- function(L, Tmat=diag(ncol(L)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-8, maxit=1000) {
+pstT <- function(L, Tmat=diag(ncol(L)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
    if(is.null(W))      stop("argument W must be specified.")
    if(is.null(Target)) stop("argument Target must be specified.")
-   z <- GPForth(L, Tmat=Tmat, method="pst", W=W, Target=Target, normalize=normalize, eps=eps, maxit=maxit)
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
+	    method="pst", methodArgs=list(W=W, Target=Target))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
-pstQ <- function(L, Tmat=diag(ncol(L)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-8, maxit=1000) {
+pstQ <- function(L, Tmat=diag(ncol(L)), W=NULL, Target=NULL, normalize=FALSE, eps=1e-5, maxit=1000) {
    if(is.null(W))      stop("argument W must be specified.")
    if(is.null(Target)) stop("argument Target must be specified.")
-   z <- GPFoblq(L, Tmat=Tmat, method="pst", W=W, Target=Target, normalize=normalize, eps=eps, maxit=maxit)
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
+	  method="pst", methodArgs=list(W=W, Target=Target))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -220,8 +273,9 @@ vgQ.pst <- function(L, W=NULL, Target=NULL){
         Method="Partially specified target")
 }
 
-oblimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000){
-   z <- GPFoblq(L, Tmat=Tmat, method="oblimax", normalize=normalize, eps=eps, maxit=maxit)
+oblimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000){
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+            method="oblimax")
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -239,18 +293,18 @@ vgQ.oblimax <- function(L){
        Method="oblimax")
 }
 
-entropy <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="entropy", normalize=normalize, eps=eps, maxit=maxit)
+entropy <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, method="entropy", normalize=normalize, eps=eps, maxit=maxit)
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
 vgQ.entropy <- function(L){
-  list(Gq= -(L*log(L^2) + L),
-       f= -sum(L^2*log(L^2))/2, 
+  list(Gq= -(L*log(L^2 + (L^2==0)) + L),
+       f= -sum(L^2*log(L^2 + (L^2==0)))/2, 
        Method="Minimum entropy")
 }
 
-quartimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
+quartimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
    z <- GPForth(L, Tmat=Tmat, method="quartimax", normalize=normalize, eps=eps, maxit=maxit)
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
@@ -261,7 +315,7 @@ vgQ.quartimax <- function(L){
        Method="Quartimax")
 }
 
-Varimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
+Varimax <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
    z <- GPForth(L, Tmat=Tmat, method="varimax", normalize=normalize, eps=eps, maxit=maxit)
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
@@ -273,8 +327,9 @@ vgQ.varimax <- function(L){
        Method="varimax")
 }
 
-simplimax <- function(L, Tmat=diag(ncol(L)), k=nrow(L), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="simplimax", k=k, normalize=normalize, eps=eps, maxit=maxit)
+simplimax <- function(L, Tmat=diag(ncol(L)), k=nrow(L), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+         method="simplimax", methodArgs=list(k=k))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -286,13 +341,15 @@ vgQ.simplimax <- function(L, k=nrow(L)){
        Method="Simplimax")
 }
 
-bentlerT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPForth(L, Tmat=Tmat, method="bentler", normalize=normalize, eps=eps, maxit=maxit)
+bentlerT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit, 
+	     method="bentler")
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
-bentlerQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="bentler", normalize=normalize, eps=eps, maxit=maxit)
+bentlerQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+           method="bentler")
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -305,8 +362,9 @@ vgQ.bentler <- function(L){
        Method="Bentler's criterion")
 }
 
-tandemI <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="tandemI", normalize=normalize, eps=eps, maxit=maxit)
+tandemI <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+            method="tandemI")
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -332,8 +390,8 @@ vgQ.tandemI <- function(L){  # Tandem Criterion, Comrey, 1967.
        Method="Tandem I")
   }
 
-tandemII <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="tandemII", normalize=normalize, eps=eps, maxit=maxit)
+tandemII <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, method="tandemII", normalize=normalize, eps=eps, maxit=maxit)
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -361,13 +419,15 @@ vgQ.tandemII <- function(L){  # Tandem Criterion, Comrey, 1967.
        Method="Tandem II")
   }
 
-geominT <- function(L, Tmat=diag(ncol(L)), delta=.01, normalize=FALSE, eps=1e-8, maxit=1000){
-   z <- GPForth(L, Tmat=Tmat, method="geomin", normalize=normalize, eps=eps, maxit=maxit)
+geominT <- function(L, Tmat=diag(ncol(L)), delta=.01, normalize=FALSE, eps=1e-5, maxit=1000){
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+           method="geomin", methodArgs=list(delta=delta))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
-geominQ <- function(L, Tmat=diag(ncol(L)), delta=.01, normalize=FALSE, eps=1e-8, maxit=1000){
-   z <- GPFoblq(L, Tmat=Tmat, method="geomin", normalize=normalize, eps=eps, maxit=maxit)
+geominQ <- function(L, Tmat=diag(ncol(L)), delta=.01, normalize=FALSE, eps=1e-5, maxit=1000){
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+            method="geomin", methodArgs=list(delta=delta))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -381,13 +441,15 @@ vgQ.geomin <- function(L, delta=.01){
        Method="Geomin")
   }
 
-cfT <- function(L, Tmat=diag(ncol(L)), kappa=0, normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPForth(L, Tmat=Tmat, method="cf", kappa=kappa, normalize=normalize, eps=eps, maxit=maxit)
+cfT <- function(L, Tmat=diag(ncol(L)), kappa=0, normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+             method="cf", methodArgs=list(kappa=kappa))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
-cfQ <- function(L, Tmat=diag(ncol(L)), kappa=0, normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="cf", kappa=kappa, normalize=normalize, eps=eps, maxit=maxit)
+cfQ <- function(L, Tmat=diag(ncol(L)), kappa=0, normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+             method="cf", methodArgs=list(kappa=kappa))
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -409,13 +471,15 @@ vgQ.cf <- function(L, kappa=0){
        Method=paste("Crawford-Ferguson:k=",kappa,sep=""))
 }
 
-infomaxT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPForth(L, Tmat=Tmat, method="infomax", normalize=normalize, eps=eps, maxit=maxit)
+infomaxT <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+            method="infomax")
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
-infomaxQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="infomax", normalize=normalize, eps=eps, maxit=maxit)
+infomaxQ <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPFoblq(L, Tmat=Tmat, normalize=normalize, eps=eps, maxit=maxit,
+            method="infomax")
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
@@ -447,8 +511,8 @@ vgQ.infomax <- function(L){
   list(Gq=Gq,f=f,Method=Method)
 }
 
-mccammon <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-8, maxit=1000) {
-   z <- GPFoblq(L, Tmat=Tmat, method="mccammon", normalize=normalize, eps=eps, maxit=maxit)
+mccammon <- function(L, Tmat=diag(ncol(L)), normalize=FALSE, eps=1e-5, maxit=1000) {
+   z <- GPForth(L, Tmat=Tmat, method="mccammon", normalize=normalize, eps=eps, maxit=maxit)
    list(loadings=z$Lh, Th=z$Th,  Table=z$Table, convergence=z$convergence)
    }
 
